'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Brain = function (_EventEmitter) {
  _inherits(Brain, _EventEmitter);

  /**
   * Represents somewhat persistent storage for the robot. Extend this.
   *
   * Returns a new Brain with no external storage.
   */

  function Brain(robot) {
    _classCallCheck(this, Brain);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Brain).call(this));

    _this.data = {
      users: {},
      _private: {}
    };
    _this.autoSave = true;

    robot.on('running', function () {
      _this.resetSaveInterval(5);
    });
    return _this;
  }

  /**
   * Public: Store key-value pair under the private namespace and extend
   * existing @data before emitting the 'loaded' event.
   *
   * Returns the instance for chaining.
   */


  _createClass(Brain, [{
    key: 'set',
    value: function set(key, value) {
      var pair = void 0;
      if (key === Object(key)) {
        pair = key;
      } else {
        pair = {};
        pair[key] = value;
      }
      // extend this.data._private
      Object.assign(this.data._private, pair);
      this.emit('loaded', this.data);
      return this;
    }

    /**
     * Public: Get value by key from the private namespace in this.data
     * or return null if not found.
     *
     * Returns the value.
     */

  }, {
    key: 'get',
    value: function get(key) {
      var ref = void 0;
      return (ref = this.data._private[key]) != null ? ref : null;
    }

    /**
     * Public: Remove value by key from the private namespace in this.data
     * if it exists.
     *
     * Returns the instance for chaining.
     */

  }, {
    key: 'remove',
    value: function remove(key) {
      if (this.data._private[key] != null) {
        delete this.data._private[key];
      }
      return this;
    }

    /**
     * Public: Emits the 'save' event so that 'brain' scripts can handle
     * persisting.
     *
     * Returns nothing.
     */

  }, {
    key: 'save',
    value: function save() {
      this.emit('save', this.data);
    }

    /**
     * Public: Emits the 'close' event so that 'brain' scripts can handle closing.
     *
     * Returns nothing.
     */

  }, {
    key: 'close',
    value: function close() {
      clearInterval(this.saveInterval);
      this.save();
      this.emit('close');
    }

    /**
     * Public: Enable or disable the automatic saving
     *
     * @param {boolean} enabled - A boolean whether to autosave or not
     *
     * Returns nothing
     */

  }, {
    key: 'setAutoSave',
    value: function setAutoSave(enabled) {
      this.autoSave = enabled;
    }

    /**
     * Public: Reset the interval between save function calls.
     *
     * @param {integer} seconds - An Integer of seconds between saves.
     *
     * Returns nothing
     */

  }, {
    key: 'resetSaveInterval',
    value: function resetSaveInterval(seconds) {
      var _this2 = this;

      if (this.saveInterval) {
        clearInterval(this.saveInterval);
      }
      this.saveInterval = setInterval(function () {
        if (_this2.autoSave) {
          return _this2.save();
        }
      }, seconds * 1000);
    }

    /**
     * Public: Merge keys loaded from a DB against the in memory representation.
     *
     * Caveats: Deeply nested structures don't merge well.
     *
     * Returns nothing
     */

  }, {
    key: 'mergeData',
    value: function mergeData(data) {
      var ref = data || {};
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = Object.keys(ref)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var key = _step.value;

          this.data[key] = ref[key];
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this.emit('loaded', this.data);
    }

    /**
     * Public: Get an Array of User objects stored in the brain.
     *
     * Returns an Array of User objects.
     */

  }, {
    key: 'users',
    value: function users() {
      return this.data.users;
    }

    /**
     * Public: Get a User object given a unique identifier.
     *
     * Returns a User instance of the specified user.
     */

  }, {
    key: 'userForId',
    value: function userForId(id, options) {
      var user = this.data.users[id];
      if (!user) {
        user = new _user2.default(id, options);
        this.data.users[id] = user;
      }
      if (options && options.room && (!user.room || user.room !== options.room)) {
        user = new _user2.default(id, options);
        this.data.users[id] = user;
      }
      return user;
    }

    /**
     * Public: Get a User object given a name.
     *
     * Returns a User instance for the user with the specified name.
     */

  }, {
    key: 'userForName',
    value: function userForName(name) {
      var result = null;
      var userName = void 0;
      var lowerName = name.toLowerCase();
      var users = this.data.users || {};
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = Object.keys(users)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var key = _step2.value;

          userName = users[key]['name'];
          if (userName != null && userName.toString().toLowerCase() === lowerName) {
            result = users[key];
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return result;
    }

    /**
     * Public: Get all users whose names match fuzzyName. Currently, match
     * means 'starts with', but this could be extended to match initials,
     * nicknames, etc.
     *
     * Returns an Array of User instances matching the fuzzy name.
     */

  }, {
    key: 'usersForRawFuzzyName',
    value: function usersForRawFuzzyName(fuzzyName) {
      var lowerFuzzyName = fuzzyName.toLowerCase();
      var ref = this.data.users || {};
      var results = [];
      var user = void 0;
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = Object.keys(ref)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var key = _step3.value;

          user = ref[key];
          if (user.name.toLowerCase().lastIndexOf(lowerFuzzyName, 0) === 0) {
            results.push(user);
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      return results;
    }

    /**
     * Public: If fuzzyName is an exact match for a user, returns an array with
     * just that user. Otherwise, returns an array of all users for which
     * fuzzyName is a raw fuzzy match (see usersForRawFuzzyName).
     *
     * Returns an Array of User instances matching the fuzzy name.
     */

  }, {
    key: 'usersForFuzzyName',
    value: function usersForFuzzyName(fuzzyName) {
      var matchedUsers = this.usersForRawFuzzyName(fuzzyName);
      var lowerFuzzyName = fuzzyName.toLowerCase();
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = matchedUsers[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var user = _step4.value;

          if (user.name.toLowerCase() === lowerFuzzyName) {
            return [user];
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return matchedUsers;
    }
  }]);

  return Brain;
}(_events.EventEmitter);

exports.default = Brain;