'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TextListener = exports.Listener = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _util = require('util');

var _async = require('async');

var async = _interopRequireWildcard(_async);

var _message = require('./message');

var _middleware = require('./middleware');

var _middleware2 = _interopRequireDefault(_middleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Listener = function () {
  /**
   * Listeners receive every message from the chat source and decide if they
   * want to act on it.
   * An identifier should be provided in the options parameter to uniquely
   * identify the listener (options.id).
   *
   * @param robot    - A Robot instance.
   * @param matcher  - A Function that determines if this listener should trigger the
   *            callback.
   * @param options  - An Object of additional parameters keyed on extension name
   *            (optional).
   * @param callback - A Function that is triggered if the incoming message matches.
   */

  function Listener(robot, matcher, options, callback) {
    _classCallCheck(this, Listener);

    this.robot = robot;
    this.matcher = matcher;
    this.options = options;
    this.callback = callback;

    if (this.matcher == null) {
      throw new Error('Missing a matcher for Listener');
    }

    if (this.callback == null) {
      this.callback = this.options;
      this.options = {};
    }

    if (this.options.id == null) {
      this.options.id = null;
    }

    if (this.callback == null || typeof this.callback !== 'function') {
      throw new Error('Missing a callback for Listener');
    }
  }

  /**
   * Public: Determines if the listener likes the content of the message. If
   * so, a Response built from the given Message is passed through all
   * registered middleware and potentially the Listener callback. Note that
   * middleware can intercept the message and prevent the callback from ever
   * being executed.
   *
   * @param message - A Message instance.
   * @param middleware - Optional Middleware object to execute before the Listener callback
   * @param cb - Optional callback Function called with a boolean of whether the matcher matched
   *
   * Returns a boolean of whether the matcher matched.
   * Returns before executing callback
   */


  _createClass(Listener, [{
    key: 'call',
    value: function call(message, middleware, cb) {
      var _this = this;

      var match = void 0;

      // middleware argument is optional
      if (cb == null && typeof middleware === 'function') {
        cb = middleware;
        middleware = undefined;
      }

      // ensure we have a Middleware object
      if (middleware == null) {
        middleware = new _middleware2.default(this.robot);
      }

      if (match = this.matcher(message)) {
        if (this.regex) {
          this.robot.logger.debug('Message \'' + message + '\' matched regex /' + (0, _util.inspect)(this.regex) + '/;\n          listener.options = ' + (0, _util.inspect)(this.options));
        }
        // special middleware-like function that always executes the Listener's
        // callback and calls done (never calls 'next')
        var executeListener = function executeListener(context, done) {
          _this.robot.logger.debug('Executing listener callback for Message \'${message}\'');
          try {
            _this.callback(context.response);
          } catch (err) {
            _this.robot.emit('error', err, context.response);
          }
          done();
        };

        // When everything is finished (down the middleware stack and back up),
        // pass control back to the robot
        var allDone = function allDone() {
          // Yes, we tried to execute the listener callback (middleware may
          // have intercepted before actually executing though)
          if (cb) {
            _middleware2.default.ticker(function () {
              return cb(true);
            });
          }
        };

        var response = new this.robot.Response(this.robot, message, match);
        middleware.execute({
          listener: this,
          response: response
        }, executeListener, allDone);
        return true;
      } else {
        if (cb) {
          // No, we didn't try to execute the listener callback
          process.nextTick(function () {
            return cb(false);
          });
        }
        return false;
      }
    }
  }]);

  return Listener;
}();

var TextListener = function (_Listener) {
  _inherits(TextListener, _Listener);

  /**
   * TextListeners receive every message from the chat source and decide if they
   * want to act on it.
   *
   * @param robot    - A Robot instance.
   * @param regex    - A Regex that determines if this listener should trigger the
   *            callback.
   * @param options  - An Object of additional parameters keyed on extension name
   *            (optional).
   * @param callback - A Function that is triggered if the incoming message matches.
   */

  function TextListener(robot, regex, options, callback) {
    _classCallCheck(this, TextListener);

    var _this2 = _possibleConstructorReturn(this, Object.getPrototypeOf(TextListener).call(this, robot, regex, options, callback));

    _this2.regex = regex;
    _this2.matcher = function (message) {
      if (message instanceof _message.TextMessage) {
        return message.match(_this2.regex);
      }
    };

    return _this2;
  }

  return TextListener;
}(Listener);

exports.Listener = Listener;
exports.TextListener = TextListener;