"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Represents an incoming message from the chat.
 *
 * @param {object} user A User instance that sent the message.
 * @param {boolean} done completed message
 */

var Message = function () {
  function Message(user) {
    var done = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

    _classCallCheck(this, Message);

    this.user = user;
    this.room = user.room;
    this.done = done ? true : false;
  }

  /**
   * Indicates that no other Listener should be called on this object.
   */


  _createClass(Message, [{
    key: "finish",
    value: function finish() {
      this.done = true;
    }
  }]);

  return Message;
}();

/**
 * Represents an incoming message from the chat.
 * @param {object} user - A User instance that sent the message.
 * @param {string} text - A String message.
 * @param {string} id   - A String of the message ID.
 */


var TextMessage = function (_Message) {
  _inherits(TextMessage, _Message);

  function TextMessage(user, text, id) {
    _classCallCheck(this, TextMessage);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(TextMessage).call(this, user));

    _this.text = text;
    _this.id = id;
    return _this;
  }

  /**
   * Determines if the message matches the given regex.
   * @param {string} regex - A Regex to check.
   *
   * Returns a Match object or null.
   */


  _createClass(TextMessage, [{
    key: "match",
    value: function match(regex) {
      return this.text.match(regex);
    }

    /**
     * String representation of a TextMessage.
     *
     * Returns the message text
     */

  }, {
    key: "toString",
    value: function toString() {
      return this.text;
    }
  }]);

  return TextMessage;
}(Message);

/**
 * Represents an incoming user entrance notification.
 * @param {object} user - A User instance for the user who entered.
 * @param {string} text - Always null.
 * @param {string} id   - A String of the message ID.
 */


var EnterMessage = function (_Message2) {
  _inherits(EnterMessage, _Message2);

  function EnterMessage() {
    _classCallCheck(this, EnterMessage);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(EnterMessage).apply(this, arguments));
  }

  return EnterMessage;
}(Message);

/**
 * Represents an incoming user exit notification.
 * @param {object} user - A User instance for the user who left.
 * @param {string} text - Always null.
 * @param {string} id   - A String of the message ID.
 */


var LeaveMessage = function (_Message3) {
  _inherits(LeaveMessage, _Message3);

  function LeaveMessage() {
    _classCallCheck(this, LeaveMessage);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(LeaveMessage).apply(this, arguments));
  }

  return LeaveMessage;
}(Message);

/**
 * Represents an incoming topic change notification.
 * @param {object} user - A User instance for the user who changed the topic.
 * @param {string} text - A String of the new topic.
 * @param {string} id   - A String of the message ID.
 */


var TopicMessage = function (_Message4) {
  _inherits(TopicMessage, _Message4);

  function TopicMessage() {
    _classCallCheck(this, TopicMessage);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(TopicMessage).apply(this, arguments));
  }

  return TopicMessage;
}(Message);

/**
 * Represents a message that no matchers matched.
 * @param {object} message - The original message.
 */


var CatchAllMessage = function (_Message5) {
  _inherits(CatchAllMessage, _Message5);

  function CatchAllMessage(message) {
    _classCallCheck(this, CatchAllMessage);

    var _this5 = _possibleConstructorReturn(this, Object.getPrototypeOf(CatchAllMessage).call(this, message.user));

    _this5.message = message;
    return _this5;
  }

  return CatchAllMessage;
}(Message);

exports.Message = Message;
exports.TextMessage = TextMessage;
exports.EnterMessage = EnterMessage;
exports.LeaveMessage = LeaveMessage;
exports.TopicMessage = TopicMessage;
exports.CatchAllMessage = CatchAllMessage;