'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _async = require('async');

var async = _interopRequireWildcard(_async);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Middleware = function () {
  _createClass(Middleware, null, [{
    key: 'ticker',

    // We use this recursively, and using nextTick recursively is deprecated in node 0.10.
    value: function ticker(cb) {
      setImmediate(cb);
    }
  }]);

  function Middleware(robot) {
    _classCallCheck(this, Middleware);

    this.robot = robot;
    this.stack = [];
  }

  /**
   * Public: Execute all middleware in order and call 'next' with the latest
   * 'done' callback if last middleware calls through. If all middleware is
   * compliant, 'done' should be called with no arguments when the entire
   * round trip is complete.
   *
   * context - context object that is passed through the middleware stack.
   *     When handling errors, this is assumed to have a `response` property.
   *
   * next(context, done) - Called when all middleware is complete (assuming
   *     all continued by calling respective 'next' functions)
   *
   * done() - Initial (final) completion callback. May be wrapped by
   *     executed middleware.
   *
   * Returns nothing
   * Returns before executing any middleware
   */


  _createClass(Middleware, [{
    key: 'execute',
    value: function execute(context, next, done) {
      var _this = this;

      if (!done) {
        done = function done() {};
      }
      /**
       * Execute a single piece of middleware and update the completion callback
       * (each piece of middleware can wrap the 'done' callback with additional
       * logic).
       */
      var executeSingleMiddleware = function executeSingleMiddleware(doneFunc, middlewareFunc, cb) {
        // Match the async.reduce interface
        var nextFunc = function nextFunc(newDoneFunc) {
          return cb(null, newDoneFunc || doneFunc);
        };
        // Catch errors in synchronous middleware
        try {
          middlewareFunc.call(undefined, context, nextFunc, doneFunc);
        } catch (err) {
          // Maintaining the existing error interface (Response object)
          _this.robot.emit('error', err, context.response);
          // Forcibly fail the middleware and stop executing deeper
          doneFunc();
        }
      };

      // Executed when the middleware stack is finished
      var allDone = function allDone(_, finalDoneFunc) {
        return next(context, finalDoneFunc);
      };

      // Execute each piece of middleware, collecting the latest 'done' callback
      // at each step.
      process.nextTick(function () {
        return async.reduce(_this.stack, done, executeSingleMiddleware, allDone);
      });
    }

    /**
     * Public: Registers new middleware
     *
     * middleware - A generic pipeline component function that can either
     *              continue the pipeline or interrupt it. The function is called
     *              with (robot, context, next, done). If execution should
     *              continue (next middleware, final callback), the middleware
     *              should call the 'next' function with 'done' as an optional
     *              argument.
     *              If not, the middleware should call the 'done' function with
     *              no arguments. Middleware may wrap the 'done' function in
     *              order to execute logic after the final callback has been
     *              executed.
     *
     * Returns nothing.
     */

  }, {
    key: 'register',
    value: function register(middleware) {
      if (middleware.length !== 3) {
        throw new Error('Incorrect number of arguments for middleware callback\n        (expected 3, got ' + middleware.length + ')');
      }
      this.stack.push(middleware);
    }
  }]);

  return Middleware;
}();

exports.default = Middleware;