'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Response = function () {
  /**
   * Public: Responses are sent to matching listeners. Messages know about the
   * content and user that made the original message, and how to reply back to
   * them.
   *
   * @param {object} robot   - A Robot instance.
   * @param {object} message - A Message instance.
   * @param {object} match   - A Match object from the successful Regex match.
   */

  function Response(robot, message, match) {
    _classCallCheck(this, Response);

    this.robot = robot;
    this.message = message;
    this.match = match;
    this.envelope = {
      room: this.message.room,
      user: this.message.user,
      message: this.message
    };
  }

  /**
   * Public: Posts a message back to the chat source
   *
   * @param {...string} strings - One or more strings to be posted. The order of these strings
   *           should be kept intact.
   *
   * Returns nothing.
   */


  _createClass(Response, [{
    key: 'send',
    value: function send() {
      for (var _len = arguments.length, strings = Array(_len), _key = 0; _key < _len; _key++) {
        strings[_key] = arguments[_key];
      }

      this.runWithMiddleware.apply(this, ['send', { plaintext: true }].concat(strings));
    }

    /**
     * Public: Posts an emote back to the chat source
     *
     * @param {...string} strings - One or more strings to be posted. The order of these strings
     *           should be kept intact.
     *
     * Returns nothing.
     */

  }, {
    key: 'emote',
    value: function emote() {
      for (var _len2 = arguments.length, strings = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        strings[_key2] = arguments[_key2];
      }

      this.runWithMiddleware.apply(this, ['emote', { plaintext: true }].concat(strings));
    }

    /**
     * Public: Posts a message mentioning the current user.
     *
     * @param {...string} strings - One or more strings to be posted. The order of these strings
     *           should be kept intact.
     *
     * Returns nothing.
     */

  }, {
    key: 'reply',
    value: function reply() {
      for (var _len3 = arguments.length, strings = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        strings[_key3] = arguments[_key3];
      }

      this.runWithMiddleware.apply(this, ['reply', { plaintext: true }].concat(strings));
    }

    /**
     * Public: Posts a topic changing message
     *
     * @param {...string} strings - One or more strings to set as the topic of the
     *           room the bot is in.
     *
     * Returns nothing.
     */

  }, {
    key: 'topic',
    value: function topic() {
      for (var _len4 = arguments.length, strings = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        strings[_key4] = arguments[_key4];
      }

      this.runWithMiddleware.apply(this, ['topic', { plaintext: true }].concat(strings));
    }

    /**
     * Public: Play a sound in the chat source
     *
     * @param {...string} strings - One or more strings to be posted as sounds to play. The order of
     *           these strings should be kept intact.
     *
     * Returns nothing.
     */

  }, {
    key: 'play',
    value: function play() {
      for (var _len5 = arguments.length, strings = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        strings[_key5] = arguments[_key5];
      }

      this.runWithMiddleware.apply(this, ['play'].concat(strings));
    }

    /**
     * Public: Posts a message in an unlogged room
     *
     * @param {...string} strings - One or more strings to be posted. The order of these strings
     *           should be kept intact.
     *
     * Returns nothing.
     */

  }, {
    key: 'locked',
    value: function locked() {
      for (var _len6 = arguments.length, strings = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        strings[_key6] = arguments[_key6];
      }

      this.runWithMiddleware.apply(this, ['locked', { plaintext: true }].concat(strings));
    }

    /**
     * Private: Call with a method for the given strings using response
     * middleware.
     */

  }, {
    key: 'runWithMiddleware',
    value: function runWithMiddleware(methodName, opts) {
      var _this = this;

      var callback = undefined;

      for (var _len7 = arguments.length, strings = Array(_len7 > 2 ? _len7 - 2 : 0), _key7 = 2; _key7 < _len7; _key7++) {
        strings[_key7 - 2] = arguments[_key7];
      }

      var copy = strings.slice(0);
      if (typeof copy[copy.length - 1] === 'function') {
        callback = copy.pop();
      }
      var context = { response: this, strings: copy, method: methodName };
      context.plaintext = opts.plaintext && opts.plaintext === true;
      var responseMiddlewareDone = function responseMiddlewareDone() {};
      var runAdapterSend = function runAdapterSend(_, done) {
        var _robot$adapter;

        var result = context.strings;
        callback && result.push(callback);
        (_robot$adapter = _this.robot.adapter)[methodName].apply(_robot$adapter, [_this.envelope].concat(_toConsumableArray(result)));
        done();
      };
      this.robot.middleware.response.execute(context, runAdapterSend, responseMiddlewareDone);
    }

    /**
     * Public: Picks a random item from the given items.
     *
     * @param {object[]} items - An Array of items.
     *
     * @return {object} A random item.
     */

  }, {
    key: 'random',
    value: function random(items) {
      return items[Math.floor(Math.random() * items.length)];
    }

    /**
     * Public: Tell the message to stop dispatching to listeners
     *
     * Returns nothing.
     */

  }, {
    key: 'finish',
    value: function finish() {
      this.message.finish();
    }

    /**
     * Public: Creates a scoped http client with chainable methods for
     * modifying the request. This doesn't actually make a request though.
     * Once your request is assembled, you can call `get()`/`post()`/etc to
     * send the request.
     *
     * @return {object} A ScopedClient instance.
     */

  }, {
    key: 'http',
    value: function http(url, options) {
      return this.robot.http(url, options);
    }
  }]);

  return Response;
}();

exports.default = Response;