'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();
// support load coffee script plugins


var _fs = require('fs');

var Fs = _interopRequireWildcard(_fs);

var _log = require('log');

var _log2 = _interopRequireDefault(_log);

var _path = require('path');

var Path = _interopRequireWildcard(_path);

var _scopedHttpClient = require('scoped-http-client');

var HttpClient = _interopRequireWildcard(_scopedHttpClient);

var _events3 = require('events');

var _async = require('async');

var async = _interopRequireWildcard(_async);

require('coffee-script/register');

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

var _brain = require('./brain');

var _brain2 = _interopRequireDefault(_brain);

var _response = require('./response');

var _response2 = _interopRequireDefault(_response);

var _listener = require('./listener');

var _message = require('./message');

var _middleware = require('./middleware');

var _middleware2 = _interopRequireDefault(_middleware);

var _server = require('./server');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var WEBBY_DEFAULT_ADAPTERS = ['shell'];

var WEBBY_DOCUMENTATION_SECTIONS = ['description', 'dependencies', 'configuration', 'commands', 'notes', 'author', 'authors', 'examples', 'tags', 'urls'];

var Robot = function () {
  /**
   * Robots receive messages from a chat source (Campfire, irc, etc), and
   * dispatch them to matching listeners.
   *
   * @param {string} adapterPath - A String of the path to built-in adapters
   *                                (defaults to src/adapters)
   * @param {string} adapter     - A String of the adapter name.
   * @param {boolean} httpd      - A Boolean whether to enable the HTTP daemon.
   * @param {string} name        - A String of the robot name,
   *                                defaults to Webby.
   *
   * Returns nothing.
   */

  function Robot(adapterPath, adapterName, httpd) {
    var _this = this;

    var name = arguments.length <= 3 || arguments[3] === undefined ? 'webby' : arguments[3];
    var alias = arguments.length <= 4 || arguments[4] === undefined ? false : arguments[4];

    _classCallCheck(this, Robot);

    if (this.adapterPath === undefined) {
      this.adapterPath = Path.join(__dirname, 'adapters');
    }
    this.name = name;
    this.events = new _events3.EventEmitter();
    this.brain = new _brain2.default(this);
    this.alias = alias;
    this.adapter = null;
    this.Response = _response2.default;
    this.commands = [];
    this.listeners = [];
    this.middleware = {
      listener: new _middleware2.default(this),
      response: new _middleware2.default(this),
      receive: new _middleware2.default(this)
    };
    this.logger = new _log2.default(process.env.WEBBY_LOG_LEVEL || 'info');
    this.pingIntervalId = null;
    this.globalHttpOptions = {};
    this.parseVersion();
    if (httpd) {
      this.router = new _server.ExpressRouter(this).router;
    } else {
      this.router = new _server.NullRouter(this).router;
    }
    this.adapterName = adapterName;
    this.loadAdapter();

    this.errorHandlers = [];
    this.on('error', function (err, res) {
      _this.invokeErrorHandlers(err, res);
    });
    this.onUncaughtException = function (err) {
      _this.emit('error', err);
    };
    process.on('uncaughtException', this.onUncaughtException);
  }

  /**
   * Public: Adds a custom Listener with the provided matcher, options, and
   * callback
   *
   * @param matcher  - A Function that determines whether to call the callback.
   *            Expected to return a truthy value if the callback should be
   *            executed.
   * @param {object} options  - An Object of additional parameters keyed on
   *                             extension name (optional).
   * @param callback - A Function that is called with a Response object if the
   *            matcher function returns true.
   *
   * Returns nothing.
   */


  _createClass(Robot, [{
    key: 'listen',
    value: function listen(matcher, options, callback) {
      this.listeners.push(new _listener.Listener(this, matcher, options, callback));
    }

    /**
     * Public: Adds a Listener that attempts to match incoming messages based on
     * a Regex.
     *
     * @param {string} regex - A Regex that determines if the callback should be
     *                          called.
     * @param {object} options  - An Object of additional parameters keyed on
     *                             extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'hear',
    value: function hear(regex, options, callback) {
      this.listeners.push(new _listener.TextListener(this, regex, options, callback));
    }

    /**
     * Public: Adds a Listener that attempts to match incoming messages directed
     * at the robot based on a Regex. All regexes treat patterns like they begin
     * with a '^'
     *
     * @param {string} regex - A Regex that determines if the callback
     *                          should be called.
     * @param {object} options - An Object of additional parameters keyed on
     *                            extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'respond',
    value: function respond(regex, options, callback) {
      this.hear(this.respondPattern(regex), options, callback);
    }

    /**
     * Public: Build a regular expression that matches messages addressed
     * directly to the robot
     *
     * @param {string} regex - A RegExp for the message part that follows the
     *                          robot's name/alias
     *
     * Returns RegExp.
     */

  }, {
    key: 'respondPattern',
    value: function respondPattern(regex) {
      var re = regex.toString().split('/');
      re.shift();
      var modifiers = re.pop();
      if (re[0] && re[0][0] === '^') {
        this.logger.warning('Anchors don\'t work well with respond, ' + 'perhaps you want to use \'hear\'');
        this.logger.warning('The regex in question was ' + regex.toString());
      }
      var pattern = re.join('/');
      var name = this.name.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
      var newRegex = void 0;
      if (this.alias) {
        var alias = this.alias.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
        var ref = name.length > alias.length ? [name, alias] : [alias, name],
            a = ref[0],
            b = ref[1];
        newRegex = new RegExp('^\\s*[@]?(?:' + a + '[:,]?|' + b + '[:,]?)\\s*(?:' + pattern + ')', modifiers);
      } else {
        newRegex = new RegExp('^\\s*[@]?' + name + '[:,]?\\s*(?:' + pattern + ')', modifiers);
      }
      return newRegex;
    }

    /**
     * Public: Adds a Listener that triggers when anyone enters the room.
     *
     * @param {object} options  - An Object of additional parameters keyed on
     *                             extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'enter',
    value: function enter(options, callback) {
      this.listen(function (msg) {
        return msg instanceof _message.EnterMessage;
      }, options, callback);
    }

    /**
     * Public: Adds a Listener that triggers when anyone leaves the room.
     *
     * @param {object} options  - An Object of additional parameters keyed on
     *                             extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'leave',
    value: function leave(options, callback) {
      this.listen(function (msg) {
        return msg instanceof _message.LeaveMessage;
      }, options, callback);
    }

    /**
     * Public: Adds a Listener that triggers when anyone changes the topic.
     *
     * @param {object} options  - An Object of additional parameters keyed on
     *                             extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'topic',
    value: function topic(options, callback) {
      this.listen(function (msg) {
        return msg instanceof _message.TopicMessage;
      }, options, callback);
    }

    /**
     * Public: Adds an error handler when an uncaught exception or user emitted
     * error event occurs.
     *
     * @param callback - A Function that is called with the error object.
     *
     * Returns nothing.
     */

  }, {
    key: 'error',
    value: function error(callback) {
      this.errorHandlers.push(callback);
    }

    /**
     * Calls and passes any registered error handlers for unhandled exceptions or
     * user emitted error events.
     *
     * @param {object} err - An Error object.
     * @param {object} res - An optional Response object that generated the error
     *
     * Returns nothing.
     */

  }, {
    key: 'invokeErrorHandlers',
    value: function invokeErrorHandlers(err, res) {
      this.logger.error(err.stack);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this.errorHandlers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var errorHandler = _step.value;

          try {
            errorHandler(err, res);
          } catch (error) {
            this.logger.error('while invoking error handler:\n          ' + error + '\n' + error.stack);
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    /**
     * Public: Adds a Listener that triggers when no other text matchers match.
     *
     * @param {object} options  - An Object of additional parameters keyed on
     *                             extension name (optional).
     * @param callback - A Function that is called with a Response object.
     *
     * Returns nothing.
     */

  }, {
    key: 'catchAll',
    value: function catchAll(options, callback) {
      if (callback == null) {
        callback = options;
        options = {};
      }
      this.listen(function (msg) {
        return msg instanceof _message.CatchAllMessage;
      }, options, function (msg) {
        msg.message = msg.message.message;
        return callback(msg);
      });
    }

    /**
     * Public: Registers new middleware for execution after matching but before
     * Listener callbacks
     *
     * @param middleware - A function that determines whether or not a given
     *         matching Listener should be executed.
     *         The function is called with (context, next, done).
     *         If execution should continue (next middleware, Listener callback),
     *         the middleware should call the 'next' function with 'done' as an
     *         argument.
     *         If not, the middleware should call the 'done' function with
     *         no arguments.
     *
     * Returns nothing.
     */

  }, {
    key: 'listenerMiddleware',
    value: function listenerMiddleware(middleware) {
      this.middleware.listener.register(middleware);
    }

    /**
     * Public: Registers new middleware for execution as a response to any
     * message is being sent.
     *
     * @param middleware - A function that examines an outgoing message and can
     *         modify it or prevent its sending. The function is called with
     *         (context, next, done). If execution should continue,
     *         the middleware should call next(done). If execution should stop,
     *         the middleware should call done(). To modify the outgoing message,
     *         set context.string to a new message.
     *
     * Returns nothing.
     */

  }, {
    key: 'responseMiddleware',
    value: function responseMiddleware(middleware) {
      this.middleware.response.register(middleware);
    }

    /**
     * Public: Registers new middleware for execution before matching
     *
     * @param middleware - A function that determines whether or not listeners
     *         should be checked.
     *         The function is called with (context, next, done).
     *         If ext, next, done). If execution should continue to the next
     *         middleware or matching phase, it should call the 'next'
     *         function with 'done' as an argument. If not, the middleware
     *         should call the 'done' function with no arguments.
     *
     * Returns nothing.
     */

  }, {
    key: 'receiveMiddleware',
    value: function receiveMiddleware(middleware) {
      this.middleware.receive.register(middleware);
    }

    /**
     * Public: Passes the given message to any interested Listeners after running
     *         receive middleware.
     *
     * @param {object} message - A Message instance. Listeners can flag this
     *              message as 'done' to prevent further execution.
     *
     * @param cb - Optional callback that is called when message processing
     *              is complete
     *
     * Returns nothing.
     * Returns before executing callback
     */

  }, {
    key: 'receive',
    value: function receive(message, cb) {
      // When everything is finished (down the middleware stack and back up),
      // pass control back to the robot
      this.middleware.receive.execute({
        response: new _response2.default(this, message)
      }, this.processListeners.bind(this), cb);
    }

    /**
     * Private: Passes the given message to any interested Listeners.
     *
     * @param {object} context - A Message instance. Listeners can flag this
     *                            message as 'done' to prevent further execution.
     *
     * @param done - Optional callback that is called when message processing is
     *                complete
     *
     * Returns nothing.
     * Returns before executing callback
     */

  }, {
    key: 'processListeners',
    value: function processListeners(context, done) {
      var _this2 = this;

      // Try executing all registered Listeners in order of registration
      // and return after message is done being processed
      var anyListenersExecuted = false;
      async.detectSeries(this.listeners, function (listener, cb) {
        try {
          listener.call(context.response.message, _this2.middleware.listener, function (listenerExecuted) {
            anyListenersExecuted = anyListenersExecuted || listenerExecuted;
            // Defer to the event loop at least after every listener so the
            // stack doesn't get too big
            _middleware2.default.ticker(function () {
              // Stop processing when message.done == true
              cb(context.response.message.done);
            });
          });
        } catch (error) {
          _this2.emit('error', error, new _response2.default(_this2, context.response.message, []));
          cb(false);
        }
      }, function () {
        if (!(context.response.message instanceof _message.CatchAllMessage) && !anyListenersExecuted) {
          _this2.logger.debug('No listeners executed; falling back to catch-all');
          _this2.receive(new _message.CatchAllMessage(context.response.message), done);
        } else {
          if (done != null) {
            process.nextTick(done);
          }
        }
      });
    }

    /**
     * Public: Loads a file in path.
     *
     * @param {string} path - A String path on the filesystem.
     * @param {string} file - A String filename in path on the filesystem.
     *
     * Returns nothing.
     */

  }, {
    key: 'loadFile',
    value: function loadFile(path, file) {
      var ext = Path.extname(file);
      var fullPath = Path.join(path, Path.basename(file, ext));
      if (require.extensions[ext]) {
        var script = void 0;
        try {
          script = require(fullPath);
          if (typeof script === 'function') {
            script(this);
            this.parseHelp(Path.join(path, file));
          } else {
            this.logger.warning('Expected ' + fullPath + '\n             to assign a function to module.exports, got ' + (typeof script === 'undefined' ? 'undefined' : _typeof(script)));
          }
        } catch (error) {
          this.logger.error('Unable to load ' + fullPath + ': ' + error.stack);
          process.exit(1);
        }
      }
    }

    /**
     * Public: Loads every script in the given path.
     *
     * @param {string} path - A String path on the filesystem.
     *
     * Returns nothing.
     */

  }, {
    key: 'load',
    value: function load(path) {
      this.logger.debug('Loading scripts from ' + path);
      if (Fs.existsSync(path)) {
        var ref = Fs.readdirSync(path).sort();
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = ref[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var file = _step2.value;

            this.loadFile(path, file);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    }

    /**
     * Public: Load scripts specified in the `hubot-scripts.json` file.
     *
     * @param {string} path    - A String path to the hubot-scripts files.
     * @param {string[]} scripts - An Array of scripts to load.
     *
     * Returns nothing.
     */

  }, {
    key: 'loadHubotScripts',
    value: function loadHubotScripts(path, scripts) {
      this.logger.debug('Loading hubot-scripts from ' + path);
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = scripts[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var script = _step3.value;

          this.loadFile(path, script);
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }

    /**
     * Public: Load scripts from packages specified in the
     * `external-scripts.json` file.
     *
     * @param {string[]} packages - An Array of packages containing hubot scripts to load.
     *
     * Returns nothing.
     */

  }, {
    key: 'loadExternalScripts',
    value: function loadExternalScripts(packages) {
      this.logger.debug('Loading external-scripts from npm packages');
      try {
        if (packages instanceof Array) {
          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = packages[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var pkg = _step4.value;

              require(pkg)(this);
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }
        } else {
          var _iteratorNormalCompletion5 = true;
          var _didIteratorError5 = false;
          var _iteratorError5 = undefined;

          try {
            for (var _iterator5 = packages[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
              var _pkg = _step5.value;

              require(_pkg)(this, packages[_pkg]);
            }
          } catch (err) {
            _didIteratorError5 = true;
            _iteratorError5 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion5 && _iterator5.return) {
                _iterator5.return();
              }
            } finally {
              if (_didIteratorError5) {
                throw _iteratorError5;
              }
            }
          }
        }
      } catch (error) {
        this.logger.error('Error loading scripts from npm package - ' + error.stack);
        process.exit(1);
      }
    }

    /**
     * Load the adapter Hubot is going to use.
     *
     * Returns nothing.
     */

  }, {
    key: 'loadAdapter',
    value: function loadAdapter() {
      this.logger.debug('Loading adapter ' + this.adapterName);
      try {
        var path = WEBBY_DEFAULT_ADAPTERS.indexOf(this.adapterName) >= 0 ? this.adapterPath + '/' + this.adapterName : 'hubot-' + this.adapterName;
        this.adapter = require(path).use(this);
      } catch (error) {
        this.logger.error('Cannot load adapter ' + this.adapterName + ' - ' + error);
        process.exit(1);
      }
    }

    /**
     * Public: Help Commands for Running Scripts.
     *
     * Returns an Array of help commands for running scripts.
     */

  }, {
    key: 'helpCommands',
    value: function helpCommands() {
      return this.commands.sort();
    }

    /**
     * Private: load help info from a loaded script.
     *
     * @param {string} path - A String path to the file on disk.
     *
     * Returns nothing.
     */

  }, {
    key: 'parseHelp',
    value: function parseHelp(path) {
      this.logger.debug('Parsing help for ' + path);
      var scriptName = Path.basename(path).replace(/\.(coffee|js)$/, '');
      var scriptDocumentation = {};
      var body = Fs.readFileSync(path, 'utf-8');
      var line = void 0,
          cleanedLine = void 0,
          currentSection = void 0,
          nextSection = void 0;
      var ref = body.split('\n');
      var _iteratorNormalCompletion6 = true;
      var _didIteratorError6 = false;
      var _iteratorError6 = undefined;

      try {
        for (var _iterator6 = ref[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
          var _line = _step6.value;

          if (!(_line[0] === '#' || _line.substr(0, 2) === '//')) {
            break;
          }
          cleanedLine = _line.replace(/^(#|\/\/)\s?/, '').trim();
          if (cleanedLine.length === 0) {
            continue;
          }
          if (cleanedLine.toLowerCase() === 'none') {
            continue;
          }
          nextSection = cleanedLine.toLowerCase().replace(':', '');
          if (WEBBY_DOCUMENTATION_SECTIONS.indexOf(nextSection) >= 0) {
            currentSection = nextSection;
            scriptDocumentation[currentSection] = [];
          } else {
            if (currentSection) {
              scriptDocumentation[currentSection].push(cleanedLine.trim());
              if (currentSection === 'commands') {
                this.commands.push(cleanedLine.trim());
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion6 && _iterator6.return) {
            _iterator6.return();
          }
        } finally {
          if (_didIteratorError6) {
            throw _iteratorError6;
          }
        }
      }
    }

    /**
     * Public: A helper send function which delegates to the adapter's send
     * function.
     *
     * @param {object} envelope   - A Object with message, room and user details.
     * @param {...string} strings - One or more Strings for each message to send.
     *
     * Returns nothing.
     */

  }, {
    key: 'send',
    value: function send(envelope) {
      var _adapter;

      for (var _len = arguments.length, strings = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        strings[_key - 1] = arguments[_key];
      }

      (_adapter = this.adapter).send.apply(_adapter, [envelope].concat(strings));
    }

    /**
     * Public: A helper reply function which delegates to the adapter's reply
     * function.
     *
     * @param {object} envelope   - A Object with message, room and user details.
     * @param {...string} strings - One or more Strings for each message to send.
     *
     * Returns nothing.
     */

  }, {
    key: 'reply',
    value: function reply(envelope) {
      var _adapter2;

      for (var _len2 = arguments.length, strings = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        strings[_key2 - 1] = arguments[_key2];
      }

      (_adapter2 = this.adapter).reply.apply(_adapter2, [envelope].concat(strings));
    }

    /**
     * Public: A helper send function to message a room that the robot is in.
     *
     * @param {string} room    - String designating the room to message.
     * @param {...string} strings - One or more Strings for each message to send.
     *
     * Returns nothing.
     */

  }, {
    key: 'messageRoom',
    value: function messageRoom(room) {
      var _adapter3;

      var envelope = { room: room };

      for (var _len3 = arguments.length, strings = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        strings[_key3 - 1] = arguments[_key3];
      }

      (_adapter3 = this.adapter).send.apply(_adapter3, [envelope].concat(strings));
    }

    /**
     * Public: A wrapper around the EventEmitter API to make usage
     * semantically better.
     *
     * @param {string} event    - The event name.
     * @param {object} listener - A Function that is called with the
     *                             event parameter when event happens.
     *
     * Returns nothing.
     */

  }, {
    key: 'on',
    value: function on(event) {
      var _events;

      for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
        args[_key4 - 1] = arguments[_key4];
      }

      (_events = this.events).on.apply(_events, [event].concat(args));
    }

    /**
     * Public: A wrapper around the EventEmitter API to make usage
     * semantically better.
     *
     * @param {string} event   - The event name.
     * @param {string[]} args... - Arguments emitted by the event
     *
     * Returns nothing.
     */

  }, {
    key: 'emit',
    value: function emit(event) {
      var _events2;

      for (var _len5 = arguments.length, args = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
        args[_key5 - 1] = arguments[_key5];
      }

      (_events2 = this.events).emit.apply(_events2, [event].concat(args));
    }

    /**
     * Public: Kick off the event loop for the adapter
     *
     * Returns nothing.
     */

  }, {
    key: 'run',
    value: function run() {
      this.emit('running');
      this.adapter.run();
    }

    /**
     * Public: Gracefully shutdown the robot process
     *
     * Returns nothing.
     */

  }, {
    key: 'shutdown',
    value: function shutdown() {
      if (this.pingIntervalId != null) {
        clearInterval(this.pingIntervalId);
      }
      process.removeListener('uncaughtException', this.onUncaughtException);
      this.adapter.close();
      this.brain.close();
    }

    /**
     * Public: The version of Webby from npm
     *
     * Returns a String of the version number.
     */

  }, {
    key: 'parseVersion',
    value: function parseVersion() {
      var pkg = require(Path.join(__dirname, '..', 'package.json'));
      this.version = pkg.version;
    }

    /**
     * Public: Creates a scoped http client with chainable methods for
     * modifying the request. This doesn't actually make a request though.
     * Once your request is assembled, you can call `get()`/`post()`/etc to
     * send the request.
     *
     * @param {string} url - String URL to access.
     * @param {object[]} options - Optional options to pass on to the client
     *
     * Examples:
     *
     *     robot.http("http://example.com")
     *       # set a single header
     *       .header('Authorization', 'bearer abcdef')
     *
     *       # set multiple headers
     *       .headers(Authorization: 'bearer abcdef', Accept: 'application/json')
     *
     *       # add URI query parameters
     *       .query(a: 1, b: 'foo & bar')
     *
     *       # make the actual request
     *       .get() (err, res, body) ->
     *         console.log body
     *
     *       # or, you can POST data
     *       .post(data) (err, res, body) ->
     *         console.log body
     *
     *    # Can also set options
     *    robot.http("https://example.com", {rejectUnauthorized: false})
     *
     * Returns a ScopedClient instance.
     */

  }, {
    key: 'http',
    value: function http(url, options) {
      return HttpClient.create(url, this.extend({}, this.globalHttpOptions, options)).header('User-Agent', 'Webby/' + this.version);
    }

    /**
     * Private: Extend obj with objects passed as additional args.
     *
     * Returns the original object with updated changes.
     */

  }, {
    key: 'extend',
    value: function extend(obj) {
      for (var _len6 = arguments.length, sources = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
        sources[_key6 - 1] = arguments[_key6];
      }

      var _iteratorNormalCompletion7 = true;
      var _didIteratorError7 = false;
      var _iteratorError7 = undefined;

      try {
        for (var _iterator7 = sources[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
          var source = _step7.value;

          Object.assign(obj, source);
        }
      } catch (err) {
        _didIteratorError7 = true;
        _iteratorError7 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion7 && _iterator7.return) {
            _iterator7.return();
          }
        } finally {
          if (_didIteratorError7) {
            throw _iteratorError7;
          }
        }
      }

      return obj;
    }
  }]);

  return Robot;
}();

exports.default = Robot;