'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NullRouter = exports.ExpressRouter = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _connectMultiparty = require('connect-multiparty');

var _connectMultiparty2 = _interopRequireDefault(_connectMultiparty);

var _scopedHttpClient = require('scoped-http-client');

var HttpClient = _interopRequireWildcard(_scopedHttpClient);

var _basicAuth2 = require('basic-auth');

var _basicAuth3 = _interopRequireDefault(_basicAuth2);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var NullRouter =
/**
 * Setup an empty router object
 *
 * returns nothing
 */
function NullRouter(robot) {
  _classCallCheck(this, NullRouter);

  var msg = 'A script has tried registering a HTTP route while the HTTP ' + 'server is disabled with --disabled-httpd.';
  var app = {
    get: function get() {
      return robot.logger.warning(msg);
    },
    post: function post() {
      return robot.logger.warning(msg);
    },
    put: function put() {
      return robot.logger.warning(msg);
    },
    delete: function _delete() {
      return robot.logger.warning(msg);
    }
  };
  return { router: app };
};

var ExpressRouter = function () {
  /**
   * Setup the Express server's defaults.
   *
   * Returns nothing.
   */

  function ExpressRouter(robot) {
    _classCallCheck(this, ExpressRouter);

    var user = process.env.EXPRESS_USER;
    var pass = process.env.EXPRESS_PASSWORD;
    var stat = process.env.EXPRESS_STATIC;
    var port = process.env.EXPRESS_PORT || process.env.PORT || 8080;
    var address = process.env.EXPRESS_BIND_ADDRESS || process.env.BIND_ADDRESS || '0.0.0.0';
    var xpoweredby = process.env.EXPRESS_XPOWEREDBY || true;

    var app = (0, _express2.default)();

    if (xpoweredby) {
      app.use(this.rewriteXPowerBy(app, 'webby/' + robot.name));
    }

    if (user && pass) {
      app.use(this.basicAuth(user, pass));
    }
    app.use(_express2.default.query());

    // configure app to use bodyParser()
    // this will let us get the data from a POST via
    // POST: {"name":"foo","color":"red"} or
    // POST: name=foo&color=red
    app.use(_bodyParser2.default.urlencoded({ extended: true }));
    app.use(_bodyParser2.default.json());
    // replacement for deprecated express.multipart/connect.multipart
    // limit to 100mb, as per the old behavior
    app.use((0, _connectMultiparty2.default)({ maxFilesSize: 100 * 1024 * 1024 }));

    if (stat) {
      app.use(_express2.default.static(stat));
    }

    this.setupHeroku(robot);

    try {
      robot.server = app.listen(port, address);
      return { router: app };
    } catch (error) {
      robot.logger.error('Error trying to start HTTP server: ' + error + '\n\n        ' + error.stack);
      process.exit(1);
    }
  }

  /**
   *  X-Power-By header rewrite middleware for use with Express 4.x.
   *
   * @param   {string}   HTTP header name
   */


  _createClass(ExpressRouter, [{
    key: 'rewriteXPowerBy',
    value: function rewriteXPowerBy(app, name) {
      return function (req, res, next) {
        // Switch off the default 'X-Powered-By: Express' header
        app.disable('x-powered-by');

        res.setHeader('X-Powered-By', name);
        next();
      };
    }

    /**
     * Simple basic auth middleware for use with Express 4.x.
     * refer https://davidbeath.com/posts/expressjs-40-basicauth.html
     *
     * @example
     * app.use('/api-requiring-auth', utils.basicAuth('username', 'password'));
     *
     * @param   {string}   username Expected username
     * @param   {string}   password Expected password
     * @returns {function} Express 4 middleware requiring the given credentials
     */

  }, {
    key: 'basicAuth',
    value: function basicAuth(username, password) {
      return function (req, res, next) {
        function unauthorized(res) {
          res.set('WWW-Authenticate', 'Basic realm=Authorization Required');
          return res.send(401);
        };

        var user = (0, _basicAuth3.default)(req);

        if (!user || !user.name || !user.pass) {
          return unauthorized(res);
        };

        if (user.name === username && user.pass === password) {
          return next();
        } else {
          return unauthorized(res);
        }
      };
    }

    /**
     * keep bot alive if runtime environment is heroku
     */

  }, {
    key: 'setupHeroku',
    value: function setupHeroku(robot) {
      var herokuUrl = process.env.HEROKU_URL;

      if (herokuUrl) {
        if (!/\/$/.test(herokuUrl)) {
          herokuUrl += '/';
        }
        setInterval(function () {
          HttpClient.create(herokuUrl + 'hubot/ping').post(function (err, res, body) {
            robot.logger.info('keep alive ping!');
          });
        }, 5 * 60 * 1000);
      }
    }
  }]);

  return ExpressRouter;
}();

exports.ExpressRouter = ExpressRouter;
exports.NullRouter = NullRouter;